@@ -123,6 +123,16 @@ static unsigned int IsJPEG(const unsigned char *magick,const size_t length)
 #define XML_MARKER  (JPEG_APP0+1)
 #define MaxBufferExtent  8192
 #define JPEG_MARKER_MAX_SIZE 65533
+#define MaxWarningCount 3
+
+/*
+  Set to 1 to use libjpeg callback for progress indication.  This is
+  not enabled by default since it outputs multiple progress
+  indications, which may be confusing for the user.  However, the
+  libjpeg method provides more detailed progress.
+*/
+#define USE_LIBJPEG_PROGRESS 0 // Use libjpeg callback for progress
+
 static const char *xmp_std_header="http://ns.adobe.com/xap/1.0/";
 
 
@@ -136,6 +146,7 @@ typedef struct _DestinationManager
 
   JOCTET
     *buffer;
+
 } DestinationManager;
 
 typedef struct _ErrorManager
@@ -143,12 +154,27 @@ typedef struct _ErrorManager
   Image
     *image;
 
+  MagickBool
+    ping;
+
   MagickBool
     completed;
 
   jmp_buf
     error_recovery;
 
+  unsigned int
+    max_warning_count;
+
+  magick_uint16_t
+    warning_counts[JMSG_LASTMSGCODE];
+
+  int
+    max_scan_number;
+
+  unsigned char
+    buffer[65537+200];
+
 } ErrorManager;
 
 typedef struct _SourceManager
@@ -199,12 +225,15 @@ typedef struct _SourceManager
 %
 */
 
+
 /*
-  Format a libjpeg warning or trace event.  Warnings are converted to
-  GraphicsMagick warning exceptions while traces are optionally
-  logged.
+  Format a libjpeg warning or trace event while decoding.  Warnings
+  are converted to GraphicsMagick warning exceptions while traces are
+  optionally logged.
+
+  JPEG message codes range from 0 to JMSG_LASTMSGCODE
 */
-static unsigned int JPEGMessageHandler(j_common_ptr jpeg_info,int msg_level)
+static void JPEGDecodeMessageHandler(j_common_ptr jpeg_info,int msg_level)
 {
   char
     message[JMSG_LENGTH_MAX];
@@ -225,26 +254,46 @@ static unsigned int JPEGMessageHandler(j_common_ptr jpeg_info,int msg_level)
   /* msg_level is -1 for warnings, 0 and up for trace messages. */
   if (msg_level < 0)
     {
+      unsigned int strikes = 0;
       /* A warning */
       (err->format_message)(jpeg_info,message);
 
+      if ((err->msg_code >= 0) &&
+          ((size_t) err->msg_code < ArraySize(error_manager->warning_counts)))
+        {
+          error_manager->warning_counts[err->msg_code]++;
+          strikes=error_manager->warning_counts[err->msg_code];
+        }
+
       if (image->logging)
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                                "[%s] JPEG Warning: \"%s\" (code=%d, "
+                                "[%s] JPEG Warning[%u]: \"%s\""
+                                " (code=%d "
                                 "parms=0x%02x,0x%02x,"
                                 "0x%02x,0x%02x,0x%02x,0x%02x,0x%02x,0x%02x)",
-                                image->filename,message,err->msg_code,
+                                image->filename,
+                                strikes,
+                                message,err->msg_code,
                                 err->msg_parm.i[0], err->msg_parm.i[1],
                                 err->msg_parm.i[2], err->msg_parm.i[3],
                                 err->msg_parm.i[4], err->msg_parm.i[5],
                                 err->msg_parm.i[6], err->msg_parm.i[7]);
         }
+      if (strikes > error_manager->max_warning_count)
+        {
+          ThrowException2(&image->exception,CorruptImageError,(char *) message,
+                          image->filename);
+          longjmp(error_manager->error_recovery,1);
+        }
+
       if ((err->num_warnings == 0) ||
           (err->trace_level >= 3))
-        ThrowBinaryException2(CorruptImageWarning,(char *) message,
-                                    image->filename);
+        ThrowException2(&image->exception,CorruptImageWarning,message,
+                        image->filename);
+      /* JWRN_JPEG_EOF - "Premature end of JPEG file" */
       err->num_warnings++;
+      return /* False */;
     }
   else
     {
@@ -257,7 +306,56 @@ static unsigned int JPEGMessageHandler(j_common_ptr jpeg_info,int msg_level)
                                 message);
         }
     }
-  return(True);
+  return /* True */;
+}
+
+static void JPEGDecodeProgressMonitor(j_common_ptr cinfo)
+{
+  ErrorManager *error_manager = (ErrorManager *) cinfo->client_data;
+  Image *image = error_manager->image;
+  const int max_scan_number = error_manager->max_scan_number;
+
+#if USE_LIBJPEG_PROGRESS
+  {
+    struct jpeg_progress_mgr *p = cinfo->progress;
+
+#if 0
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Progress: pass_counter=%ld, pass_limit=%ld,"
+                          " completed_passes=%d, total_passes=%d, filename=%s",
+                          p->pass_counter, p->pass_limit,
+                          p->completed_passes, p->total_passes, image->filename);
+#endif
+
+    if (QuantumTick(p->pass_counter,p->pass_limit))
+      if (!MagickMonitorFormatted(p->pass_counter,p->pass_limit,&image->exception,
+                                  "[%s] Loading image: %lux%lu (pass %d of %d)...  ",
+                                  image->filename,
+                                  image->columns,image->rows,
+                                  p->completed_passes+1, p->total_passes))
+        {
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                "Quitting due to progress monitor");
+          longjmp(error_manager->error_recovery,1);
+        }
+  }
+#endif /* USE_LIBJPEG_PROGRESS */
+
+  if (cinfo->is_decompressor)
+    {
+      int scan_no = ((j_decompress_ptr) cinfo)->input_scan_number;
+
+      if (scan_no > max_scan_number)
+        {
+          char message[MaxTextExtent];
+          FormatString(message,"Scan number %d exceeds maximum scans (%d)",
+                       scan_no, max_scan_number);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s", message);
+          ThrowException2(&image->exception,CorruptImageError,(char *) message,
+                          image->filename);
+          longjmp(error_manager->error_recovery,1);
+        }
+    }
 }
 
 static boolean FillInputBuffer(j_decompress_ptr cinfo)
@@ -282,10 +380,12 @@ static boolean FillInputBuffer(j_decompress_ptr cinfo)
   return(TRUE);
 }
 
-static unsigned int GetCharacter(j_decompress_ptr jpeg_info)
+static int GetCharacter(j_decompress_ptr jpeg_info)
 {
   if (jpeg_info->src->bytes_in_buffer == 0)
-    (void) (*jpeg_info->src->fill_input_buffer)(jpeg_info);
+    if ((!((*jpeg_info->src->fill_input_buffer)(jpeg_info))) ||
+        (jpeg_info->src->bytes_in_buffer == 0))
+      return EOF;
   jpeg_info->src->bytes_in_buffer--;
   return(GETJOCTET(*jpeg_info->src->next_input_byte++));
 }
@@ -374,10 +474,7 @@ static boolean ReadComment(j_decompress_ptr jpeg_info)
   if (length <= 2)
     return(True);
   length-=2;
-  comment=MagickAllocateMemory(char *,length+1);
-  if (comment == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
-      (char *) NULL);
+  comment=(char *) error_manager->buffer;
   /*
     Read comment.
   */
@@ -389,7 +486,6 @@ static boolean ReadComment(j_decompress_ptr jpeg_info)
     }
   *p='\0';
   (void) SetImageAttribute(image,"comment",comment);
-  MagickFreeMemory(comment);
   return(True);
 }
 
@@ -445,10 +541,7 @@ static boolean ReadGenericProfile(j_decompress_ptr jpeg_info)
   /*
     Copy profile from JPEG to allocated memory.
   */
-  profile=MagickAllocateMemory(unsigned char *,length);
-  if (profile == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
-                         (char *) NULL);
+  profile=error_manager->buffer;
 
   for (i=0 ; i < length ; i++)
     profile[i]=GetCharacter(jpeg_info);
@@ -477,10 +570,12 @@ static boolean ReadGenericProfile(j_decompress_ptr jpeg_info)
   */
   status=AppendImageProfile(image,profile_name,profile+header_length,
                             length-header_length);
-  MagickFreeMemory(profile);
 
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Profile: %s, %lu bytes",
-                        profile_name, (unsigned long) header_length);
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "Profile: %s, header %" MAGICK_SIZE_T_F "u bytes, "
+                        "data %" MAGICK_SIZE_T_F "u bytes",
+                        profile_name, (MAGICK_SIZE_T) header_length,
+                        (MAGICK_SIZE_T) length-header_length);
 
   return (status);
 }
@@ -537,10 +632,7 @@ static boolean ReadICCProfile(j_decompress_ptr jpeg_info)
   /*
     Read color profile.
   */
-  profile=MagickAllocateMemory(unsigned char *,(size_t) length);
-  if (profile == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
-      (char *) NULL);
+  profile=error_manager->buffer;
 
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                         "ICC profile chunk: %ld bytes",
@@ -551,8 +643,6 @@ static boolean ReadICCProfile(j_decompress_ptr jpeg_info)
 
   (void) AppendImageProfile(image,"ICM",profile,length);
 
-  MagickFreeMemory(profile);
-
   return(True);
 }
 
@@ -639,10 +729,10 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
   if (length <= 0)
     return(True);
 
-  profile=MagickAllocateMemory(unsigned char *,(size_t) length+tag_length);
-  if (profile == (unsigned char *) NULL)
+  if ((size_t) length+tag_length > sizeof(error_manager->buffer))
     ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
       (char *) NULL);
+  profile=error_manager->buffer;
   /*
     Read the payload of this binary data.
   */
@@ -655,7 +745,6 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
 
   (void) AppendImageProfile(image,"IPTC",profile,length);
 
-  MagickFreeMemory(profile);
   return(True);
 }
 
@@ -988,12 +1077,12 @@ IsITUFax(const Image* image)
 static Image *ReadJPEGImage(const ImageInfo *image_info,
                             ExceptionInfo *exception)
 {
-  ErrorManager
-    error_manager;
-
   Image
     *image;
 
+  ErrorManager
+    error_manager;
+
   IndexPacket
     index;
 
@@ -1006,15 +1095,21 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   JSAMPROW
     scanline[1];
 
+  const char
+    *value;
+
   register long
     i;
 
-  struct jpeg_decompress_struct
-    jpeg_info;
-
   struct jpeg_error_mgr
     jpeg_error;
 
+  struct jpeg_progress_mgr
+    jpeg_progress;
+
+  struct jpeg_decompress_struct
+    jpeg_info;
+
   register JSAMPLE
     *p;
 
@@ -1043,13 +1138,24 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     Initialize structures.
   */
   (void) memset(&error_manager,0,sizeof(error_manager));
+  (void) memset(&jpeg_progress,0,sizeof(jpeg_progress));
   (void) memset(&jpeg_info,0,sizeof(jpeg_info));
   (void) memset(&jpeg_error,0,sizeof(jpeg_error));
   jpeg_info.err=jpeg_std_error(&jpeg_error);
-  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGMessageHandler;
+  jpeg_info.err->emit_message=/*(void (*)(j_common_ptr,int))*/ JPEGDecodeMessageHandler;
   jpeg_info.err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
   jpeg_pixels=(JSAMPLE *) NULL;
   error_manager.image=image;
+  error_manager.ping=image_info->ping;
+  error_manager.max_scan_number=100;
+  error_manager.max_warning_count=MaxWarningCount;
+
+  /*
+    Allow the user to set how many warnings of any given type are
+    allowed before promotion of the warning to a hard error.
+  */
+  if ((value=AccessDefinition(image_info,"jpeg","max-warnings")))
+    error_manager.max_warning_count=strtol(value,(char **) NULL, 10);
 
   /*
     Set initial longjmp based error handler.
@@ -1067,6 +1173,22 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   jpeg_info.client_data=(void *) &error_manager;
 
   jpeg_create_decompress(&jpeg_info);
+  /*
+    Specify a memory limit for libjpeg which is 1/5th the absolute
+    limit.  Don't actually consume the resource since we don't know
+    how much libjpeg will actually consume.
+  */
+  jpeg_info.mem->max_memory_to_use=(long) (GetMagickResourceLimit(MemoryResource) -
+                                           GetMagickResource(MemoryResource))/5U;
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Memory capped to %ld bytes", jpeg_info.mem->max_memory_to_use);
+  /*
+    Register our progress monitor
+  */
+  jpeg_progress.progress_monitor=(void (*)(j_common_ptr)) JPEGDecodeProgressMonitor;
+  jpeg_info.progress=&jpeg_progress;
+
   JPEGSourceManager(&jpeg_info,image);
   jpeg_set_marker_processor(&jpeg_info,JPEG_COM,ReadComment);
   jpeg_set_marker_processor(&jpeg_info,ICC_MARKER,ReadICCProfile);
@@ -1074,7 +1196,13 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   for (i=1; i < 16; i++)
     if ((i != 2) && (i != 13) && (i != 14))
       jpeg_set_marker_processor(&jpeg_info,JPEG_APP0+i,ReadGenericProfile);
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Reading JPEG header...");
   i=jpeg_read_header(&jpeg_info,True);
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Done with reading JPEG header");
   if (IsITUFax(image))
     {
       if (image->logging)
@@ -1111,6 +1239,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
                                   ResolutionTypeToString(image->units));
         }
     }
+
   /*
     If the desired image size is pre-set (e.g. by using -size), then
     let the JPEG library subsample for us.
@@ -1183,52 +1312,63 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   image->compression=JPEGCompression;
   image->interlace=LineInterlace;
 #endif
-  {
-    const char
-      *value;
 
-    /*
-      Allow the user to enable/disable block smoothing.
-    */
-    if ((value=AccessDefinition(image_info,"jpeg","block-smoothing")))
-      {
-        if (LocaleCompare(value,"FALSE") == 0)
-          jpeg_info.do_block_smoothing=False;
-        else
-          jpeg_info.do_block_smoothing=True;
-      }
+  /*
+    Allow the user to enable/disable block smoothing.
+  */
+  if ((value=AccessDefinition(image_info,"jpeg","block-smoothing")))
+    {
+      if (LocaleCompare(value,"FALSE") == 0)
+        jpeg_info.do_block_smoothing=False;
+      else
+        jpeg_info.do_block_smoothing=True;
+    }
 
-    /*
-      Allow the user to select the DCT decoding algorithm.
-    */
-    if ((value=AccessDefinition(image_info,"jpeg","dct-method")))
-      {
-        if (LocaleCompare(value,"ISLOW") == 0)
-          jpeg_info.dct_method=JDCT_ISLOW;
-        else if (LocaleCompare(value,"IFAST") == 0)
-          jpeg_info.dct_method=JDCT_IFAST;
-        else if (LocaleCompare(value,"FLOAT") == 0)
-          jpeg_info.dct_method=JDCT_FLOAT;
-        else if (LocaleCompare(value,"DEFAULT") == 0)
-          jpeg_info.dct_method=JDCT_DEFAULT;
-        else if (LocaleCompare(value,"FASTEST") == 0)
-          jpeg_info.dct_method=JDCT_FASTEST;
-      }
+  /*
+    Allow the user to select the DCT decoding algorithm.
+  */
+  if ((value=AccessDefinition(image_info,"jpeg","dct-method")))
+    {
+      if (LocaleCompare(value,"ISLOW") == 0)
+        jpeg_info.dct_method=JDCT_ISLOW;
+      else if (LocaleCompare(value,"IFAST") == 0)
+        jpeg_info.dct_method=JDCT_IFAST;
+      else if (LocaleCompare(value,"FLOAT") == 0)
+        jpeg_info.dct_method=JDCT_FLOAT;
+      else if (LocaleCompare(value,"DEFAULT") == 0)
+        jpeg_info.dct_method=JDCT_DEFAULT;
+      else if (LocaleCompare(value,"FASTEST") == 0)
+        jpeg_info.dct_method=JDCT_FASTEST;
+    }
 
-    /*
-      Allow the user to enable/disable fancy upsampling.
-    */
-    if ((value=AccessDefinition(image_info,"jpeg","fancy-upsampling")))
-      {
-        if (LocaleCompare(value,"FALSE") == 0)
-          jpeg_info.do_fancy_upsampling=False;
-        else
-          jpeg_info.do_fancy_upsampling=True;
-      }
-  }
-  (void) jpeg_start_decompress(&jpeg_info);
+  /*
+    Allow the user to enable/disable fancy upsampling.
+  */
+  if ((value=AccessDefinition(image_info,"jpeg","fancy-upsampling")))
+    {
+      if (LocaleCompare(value,"FALSE") == 0)
+        jpeg_info.do_fancy_upsampling=False;
+      else
+        jpeg_info.do_fancy_upsampling=True;
+    }
+
+  /*
+    Allow the user to adjust the maximum JPEG scan number
+  */
+  if ((value=AccessDefinition(image_info,"jpeg","max-scan-number")))
+    {
+      error_manager.max_scan_number=strtol(value,(char **) NULL, 10);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "JPEG max-scan-number set to %d",
+                            error_manager.max_scan_number);
+    }
+
+  jpeg_calc_output_dimensions(&jpeg_info);
   image->columns=jpeg_info.output_width;
   image->rows=jpeg_info.output_height;
+  image->storage_class = jpeg_info.output_components == 1 ? PseudoClass : DirectClass;
+  image->depth=Min(jpeg_info.data_precision,QuantumDepth);
+
   if (image->logging)
     {
       if (image->interlace == LineInterlace)
@@ -1252,6 +1392,25 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
                             (jpeg_info.do_block_smoothing ? "true" : "false"));
     }
 
+  if (image_info->ping)
+    {
+      jpeg_destroy_decompress(&jpeg_info);
+      CloseBlob(image);
+      return(image);
+    }
+
+  if (CheckImagePixelLimits(image, exception) != MagickPass)
+    {
+      jpeg_destroy_decompress(&jpeg_info);
+      ThrowReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+    }
+
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Starting JPEG decompression...");
+  (void) jpeg_start_decompress(&jpeg_info);
+  image->columns=jpeg_info.output_width;
+  image->rows=jpeg_info.output_height;
   {
     char
       attribute[MaxTextExtent];
@@ -1286,12 +1445,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
         jpeg_destroy_decompress(&jpeg_info);
         ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
       }
-  if (image_info->ping)
-    {
-      jpeg_destroy_decompress(&jpeg_info);
-      CloseBlob(image);
-      return(image);
-    }
   if (CheckImagePixelLimits(image, exception) != MagickPass)
     {
       jpeg_destroy_decompress(&jpeg_info);
@@ -1342,6 +1495,9 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       jpeg_destroy_decompress(&jpeg_info);
       ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     }
+  (void) memset(jpeg_pixels,0,MagickArraySize(jpeg_info.output_components,
+                                              MagickArraySize(image->columns,
+                                                              sizeof(JSAMPLE))));
 
   /*
     Extended longjmp-based error handler (with jpeg_pixels)
@@ -1462,6 +1618,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
           status=MagickFail;
           break;
         }
+#if !USE_LIBJPEG_PROGRESS
       if (QuantumTick(y,image->rows))
         if (!MagickMonitorFormatted(y,image->rows,exception,LoadImageText,
                                     image->filename,
@@ -1471,6 +1628,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
             jpeg_abort_decompress(&jpeg_info);
             break;
           }
+#endif /* !USE_LIBJPEG_PROGRESS */
     }
   /*
     Free jpeg resources.
@@ -1955,6 +2113,124 @@ static void JPEGDestinationManager(j_compress_ptr cinfo,Image * image)
   destination->image=image;
 }
 
+/*
+  Format a libjpeg warning or trace event while encoding.  Warnings
+  are converted to GraphicsMagick warning exceptions while traces are
+  optionally logged.
+
+  JPEG message codes range from 0 to JMSG_LASTMSGCODE
+*/
+static void JPEGEncodeMessageHandler(j_common_ptr jpeg_info,int msg_level)
+{
+  char
+    message[JMSG_LENGTH_MAX];
+
+  struct jpeg_error_mgr
+    *err;
+
+  ErrorManager
+    *error_manager;
+
+  Image
+    *image;
+
+  message[0]='\0';
+  err=jpeg_info->err;
+  error_manager=(ErrorManager *) jpeg_info->client_data;
+  image=error_manager->image;
+  /* msg_level is -1 for warnings, 0 and up for trace messages. */
+  if (msg_level < 0)
+    {
+      unsigned int strikes = 0;
+      /* A warning */
+      (err->format_message)(jpeg_info,message);
+
+      if ((err->msg_code >= 0) &&
+          ((size_t) err->msg_code < ArraySize(error_manager->warning_counts)))
+        {
+          error_manager->warning_counts[err->msg_code]++;
+          strikes=error_manager->warning_counts[err->msg_code];
+        }
+
+      if (image->logging)
+        {
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                "[%s] JPEG Warning[%u]: \"%s\""
+                                " (code=%d "
+                                "parms=0x%02x,0x%02x,"
+                                "0x%02x,0x%02x,0x%02x,0x%02x,0x%02x,0x%02x)",
+                                image->filename,
+                                strikes,
+                                message,err->msg_code,
+                                err->msg_parm.i[0], err->msg_parm.i[1],
+                                err->msg_parm.i[2], err->msg_parm.i[3],
+                                err->msg_parm.i[4], err->msg_parm.i[5],
+                                err->msg_parm.i[6], err->msg_parm.i[7]);
+        }
+      /*
+      if (strikes > error_manager->max_warning_count)
+        {
+          ThrowException2(&image->exception,CorruptImageError,(char *) message,
+                          image->filename);
+          longjmp(error_manager->error_recovery,1);
+        }
+
+      if ((err->num_warnings == 0) ||
+          (err->trace_level >= 3))
+        ThrowException2(&image->exception,CorruptImageWarning,message,
+                        image->filename);
+      */
+      /* JWRN_JPEG_EOF - "Premature end of JPEG file" */
+      err->num_warnings++;
+      return /* False */;
+    }
+  else
+    {
+      /* A trace message */
+      if ((image->logging) && (msg_level >= err->trace_level))
+        {
+          (err->format_message)(jpeg_info,message);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                "[%s] JPEG Trace: \"%s\"",image->filename,
+                                message);
+        }
+    }
+  return /* True */;
+}
+
+
+static void JPEGEncodeProgressMonitor(j_common_ptr cinfo)
+{
+#if USE_LIBJPEG_PROGRESS
+  struct jpeg_progress_mgr *p = cinfo->progress;
+  ErrorManager *error_manager = (ErrorManager *) cinfo->client_data;
+  Image *image = error_manager->image;
+
+#if 0
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "Progress: pass_counter=%ld, pass_limit=%ld,"
+                        " completed_passes=%d, total_passes=%d, filename=%s",
+                        p->pass_counter, p->pass_limit,
+                        p->completed_passes, p->total_passes, image->filename);
+#endif
+
+  if (QuantumTick(p->pass_counter,p->pass_limit))
+    if (!MagickMonitorFormatted(p->pass_counter,p->pass_limit,&image->exception,
+                                "[%s] Saving image: %lux%lu (pass %d of %d)...  ",
+                                image->filename,
+                                image->columns,image->rows,
+                                p->completed_passes+1, p->total_passes))
+      {
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Quitting due to progress monitor");
+        longjmp(error_manager->error_recovery,1);
+      }
+#else
+  (void) cinfo;
+#endif /* USE_LIBJPEG_PROGRESS */
+}
+
+
 static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
 {
   Image
@@ -1974,8 +2250,11 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
     scanline[1];
 
   char
-   *sampling_factors,
-   *preserve_settings;
+    *sampling_factors,
+    *preserve_settings;
+
+  const char
+    *value;
 
   long
     y;
@@ -1990,12 +2269,15 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
     i,
     x;
 
-  struct jpeg_compress_struct
-    jpeg_info;
-
   struct jpeg_error_mgr
     jpeg_error;
 
+  struct jpeg_progress_mgr
+    jpeg_progress;
+
+  struct jpeg_compress_struct
+    jpeg_info;
+
   MagickPassFail
     status;
 
@@ -2023,6 +2305,7 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
     ThrowWriterException(FileOpenError,UnableToOpenFile,imagev);
 
   (void) memset(&error_manager,0,sizeof(error_manager));
+  (void) memset(&jpeg_progress,0,sizeof(jpeg_progress));
   (void) memset(&jpeg_info,0,sizeof(jpeg_info));
   (void) memset(&jpeg_error,0,sizeof(jpeg_error));
 
@@ -2031,9 +2314,16 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
   */
   jpeg_info.client_data=(void *) imagev;
   jpeg_info.err=jpeg_std_error(&jpeg_error);
-  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGMessageHandler;
+  jpeg_info.err->emit_message=/*(void (*)(j_common_ptr,int))*/ JPEGEncodeMessageHandler;
   jpeg_info.err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
   error_manager.image=imagev;
+  error_manager.max_warning_count=MaxWarningCount;
+  /*
+    Allow the user to set how many warnings of any given type are
+    allowed before promotion of the warning to a hard error.
+  */
+  if ((value=AccessDefinition(image_info,"jpeg","max-warnings")))
+    error_manager.max_warning_count=strtol(value,(char **) NULL, 10);
   jpeg_info.client_data=(void *) &error_manager;
   if (setjmp(error_manager.error_recovery))
     {
@@ -2082,6 +2372,12 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
   jpeg_info.input_components=3;
   jpeg_info.in_color_space=JCS_RGB;
 
+  /*
+    Register our progress monitor
+  */
+  jpeg_progress.progress_monitor=(void (*)(j_common_ptr)) JPEGEncodeProgressMonitor;
+  jpeg_info.progress=&jpeg_progress;
+
   /*
     Set JPEG colorspace as per user request.
   */
@@ -2573,11 +2869,13 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
                     }
                 }
               (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
+#if !USE_LIBJPEG_PROGRESS
               if (QuantumTick(y,image->rows))
                 if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                             SaveImageText,image->filename,
                                             image->columns,image->rows))
                   break;
+#endif /* !USE_LIBJPEG_PROGRESS */
             }
         }
       else
@@ -2604,11 +2902,13 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
                     p++;
                   }
                 (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
+#if !USE_LIBJPEG_PROGRESS
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                               SaveImageText,image->filename,
                                               image->columns,image->rows))
                     break;
+#endif /* !USE_LIBJPEG_PROGRESS */
               }
           }
         else
@@ -2636,11 +2936,13 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
                     p++;
                   }
                 (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
+#if !USE_LIBJPEG_PROGRESS
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                               SaveImageText,image->filename,
                                               image->columns,image->rows))
                     break;
+#endif /* !USE_LIBJPEG_PROGRESS */
               }
           }
     }
@@ -2675,11 +2977,13 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
                   }
               }
             (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
+#if !USE_LIBJPEG_PROGRESS
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                           SaveImageText,image->filename,
                                           image->columns,image->rows))
                 break;
+#endif /* !USE_LIBJPEG_PROGRESS */
           }
       }
     else
@@ -2705,11 +3009,13 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
                   p++;
                 }
               (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
+#if !USE_LIBJPEG_PROGRESS
               if (QuantumTick(y,image->rows))
                 if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                             SaveImageText,image->filename,
                                             image->columns,image->rows))
                   break;
+#endif /* !USE_LIBJPEG_PROGRESS */
             }
         }
       else
@@ -2737,11 +3043,13 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
                   p++;
                 }
               (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
+#if !USE_LIBJPEG_PROGRESS
               if (QuantumTick(y,image->rows))
                 if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                             SaveImageText,image->filename,
                                             image->columns,image->rows))
                   break;
+#endif /* !USE_LIBJPEG_PROGRESS */
             }
         }
   if (image->logging)
