@@ -705,7 +705,7 @@ static void output_switches(Image *image,struct locale_str *locstr, int indent,
         if (flag)
           {
             FormatString(message, "\n%*scase '%c':  case '%c':\n",
-			 indent, "", tolower((int) *xl->name), toupper((int) *xl->name));
+                         indent, "", tolower((int) *xl->name), toupper((int) *xl->name));
             (void) WriteBlobString(image,message);
           }
 
@@ -881,7 +881,7 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
       WriteBlobStringWithEOL(image,"#ifndef _LOCAL_C_H");
       WriteBlobStringWithEOL(image,"#define _LOCAL_C_H");
       WriteBlobStringEOL(image);
-      WriteBlobStringWithEOL(image,"extern MagickExport const char *GetLocaleMessageFromID(const int);");
+      WriteBlobStringWithEOL(image,"extern MagickExport const char *GetLocaleMessageFromID(const int) MAGICK_FUNC_CONST;");
       WriteBlobStringEOL(image);
       FormatString(text, "#define MAX_LOCALE_MSGS %d",count);
       WriteBlobStringWithEOL(image,text);
@@ -1114,4 +1114,3 @@ static unsigned int WriteLOCALEImage(const ImageInfo *image_info,Image *image)
   CloseBlob(image);
   return(True);
 }
-
