@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2012 GraphicsMagick Group
+% Copyright (C) 2003-2018 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -172,18 +172,27 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBool
     use_crop_box = MagickFalse;
 
+  MagickBool
+    pdf_stop_on_error = MagickFalse;
+
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  
-  if ((value=AccessDefinition(image_info,"pdf","use-cropbox"))) 
+
+  if ((value=AccessDefinition(image_info,"pdf","use-cropbox")))
     {
       if (strcmp(value,"true") == 0)
-	use_crop_box = True;
+        use_crop_box = True;
     }
 
+   if ((value=AccessDefinition(image_info,"pdf","stop-on-error")))
+     {
+       if (strcmp(value,"true") == 0)
+         pdf_stop_on_error = True;
+     }
+
   /*
     Select Postscript delegate driver
   */
@@ -210,7 +219,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   dy_resolution=72.0;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
-      (void) strcpy(density,PSDensityGeometry);
+      (void) strlcpy(density,PSDensityGeometry,sizeof(density));
       count=GetMagickDimension(density,&image->x_resolution,&image->y_resolution,NULL,NULL);
       if (count != 2)
         image->y_resolution=image->x_resolution;
@@ -256,87 +265,87 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) memset(&box,0,sizeof(RectangleInfo));
     for (p=command; ; )
       {
-	c=ReadBlobByte(image);
-	if (c == EOF)
-	  break;
-	(void) fputc(c,file);
-	*p++=(char) c;
-	if ((c != '\n') && (c != '\r') && ((p-command) < (MaxTextExtent-1)))
-	  continue;
-	*p='\0';
-	p=command;
-	/*
-	  Continue unless this is a MediaBox statement.
-	*/
-	if (LocaleNCompare(command,"/Rotate ",8) == 0)
-	  {
-	    count=sscanf(command,"/Rotate %d",&rotate);
-	    if (count > 0)
-	      {
-		(void) LogMagickEvent(CoderEvent,GetMagickModule(),
-				      "Rotate by %d degrees",rotate);
-	      }
-	  }
-	q=strstr(command,MediaBox);
-	if (q == (char *) NULL)
-	  continue;
-	count=sscanf(q,"/MediaBox [%lf %lf %lf %lf",&bounds.x1,&bounds.y1,
-		     &bounds.x2,&bounds.y2);
-	if (count != 4)
-	  count=sscanf(q,"/MediaBox[%lf %lf %lf %lf",&bounds.x1,&bounds.y1,
-		       &bounds.x2,&bounds.y2);
-	if (count == 4)
-	  {
-	    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-				  "Parsed: MediaBox %lf %lf %lf %lf",
-				  bounds.x1,bounds.y1,
-				  bounds.x2,bounds.y2);
-	  }
-	if (count != 4)
-	  continue;
-	if ((bounds.x1 > bounds.x2) || (bounds.y1 > bounds.y2))
-	  continue;
-	/*
-	  Set Postscript render geometry.
-	*/
-	width=(unsigned long) (bounds.x2-bounds.x1+0.5);
-	height=(unsigned long) (bounds.y2-bounds.y1+0.5);
-	if ((width <= box.width) && (height <= box.height))
-	  continue;
-	page.width=width;
-	page.height=height;
-	box=page;
+        c=ReadBlobByte(image);
+        if (c == EOF)
+          break;
+        (void) fputc(c,file);
+        *p++=(char) c;
+        if ((c != '\n') && (c != '\r') && ((p-command) < (MaxTextExtent-1)))
+          continue;
+        *p='\0';
+        p=command;
+        /*
+          Continue unless this is a MediaBox statement.
+        */
+        if (LocaleNCompare(command,"/Rotate ",8) == 0)
+          {
+            count=sscanf(command,"/Rotate %d",&rotate);
+            if (count > 0)
+              {
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                      "Rotate by %d degrees",rotate);
+              }
+          }
+        q=strstr(command,MediaBox);
+        if (q == (char *) NULL)
+          continue;
+        count=sscanf(q,"/MediaBox [%lf %lf %lf %lf",&bounds.x1,&bounds.y1,
+                     &bounds.x2,&bounds.y2);
+        if (count != 4)
+          count=sscanf(q,"/MediaBox[%lf %lf %lf %lf",&bounds.x1,&bounds.y1,
+                       &bounds.x2,&bounds.y2);
+        if (count == 4)
+          {
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "Parsed: MediaBox %lf %lf %lf %lf",
+                                  bounds.x1,bounds.y1,
+                                  bounds.x2,bounds.y2);
+          }
+        if (count != 4)
+          continue;
+        if ((bounds.x1 > bounds.x2) || (bounds.y1 > bounds.y2))
+          continue;
+        /*
+          Set Postscript render geometry.
+        */
+        width=(unsigned long) (bounds.x2-bounds.x1+0.5);
+        height=(unsigned long) (bounds.y2-bounds.y1+0.5);
+        if ((width <= box.width) && (height <= box.height))
+          continue;
+        page.width=width;
+        page.height=height;
+        box=page;
       }
     /*
       If page is rotated right or left, then swap width and height values.
     */
     if ((90 == AbsoluteValue(rotate)) || (270 == AbsoluteValue(rotate)))
       {
-	double
-	  value;
+        double
+          value;
 
-	value=page.width;
-	page.width=page.height;
-	page.height=value;
+        value=page.width;
+        page.width=page.height;
+        page.height=value;
       }
     if ((page.width == 0) || (page.height == 0))
       {
-	SetGeometry(image,&page);
-	(void) GetGeometry(PSPageGeometry,&page.x,&page.y,&page.width,
-			   &page.height);
+        SetGeometry(image,&page);
+        (void) GetGeometry(PSPageGeometry,&page.x,&page.y,&page.width,
+                           &page.height);
       }
     if (image_info->page != (char *) NULL)
       (void) GetGeometry(image_info->page,&page.x,&page.y,&page.width,
-			 &page.height);
+                         &page.height);
     geometry[0]='\0';
     FormatString(geometry,"%lux%lu",
-		 (unsigned long) ceil(page.width*image->x_resolution/dx_resolution-0.5),
-		 (unsigned long) ceil(page.height*image->y_resolution/dy_resolution-0.5));
+                 (unsigned long) ceil(page.width*image->x_resolution/dx_resolution-0.5),
+                 (unsigned long) ceil(page.height*image->y_resolution/dy_resolution-0.5));
     if (ferror(file))
       {
-	(void) fclose(file);
-	ThrowReaderException(CorruptImageError,AnErrorHasOccurredWritingToFile,
-			     image);
+        (void) fclose(file);
+        ThrowReaderException(CorruptImageError,AnErrorHasOccurredWritingToFile,
+                             image);
       }
   }
 
@@ -347,34 +356,54 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   {
     char
-      options[MaxTextExtent];
+      options[MaxTextExtent],
+      arg[MaxTextExtent];
 
     options[0]='\0';
 
     if (use_crop_box)
-      FormatString(options,"-dUseCropBox");
+      (void) strlcat(options,"-dUseCropBox",sizeof(options));
+
+    if (pdf_stop_on_error)
+      {
+        if (options[0] != '\0')
+          (void) strlcat(options," ",sizeof(options));
+        (void) strlcat(options,"-dPDFSTOPONERROR",sizeof(options));
+      }
+
     /*
       Append subrange.
     */
     if (image_info->subrange != 0)
-      FormatString(options+strlen(options)," -dFirstPage=%lu -dLastPage=%lu",
-		   image_info->subimage+1,image_info->subimage+image_info->subrange);
+      {
+        FormatString(arg,"-dFirstPage=%lu -dLastPage=%lu",
+                     image_info->subimage+1,
+                     image_info->subimage+image_info->subrange);
+        if (options[0] != '\0')
+          (void) strlcat(options," ",sizeof(options));
+        (void) strlcat(options,arg,sizeof(options));
+      }
+
     /*
       Append authentication string.
     */
     if (image_info->authenticate != (char *) NULL)
-      FormatString(options+strlen(options)," -sPDFPassword=%.1024s",
-		   image_info->authenticate);
+      {
+        FormatString(arg,"-sPDFPassword=%.1024s", image_info->authenticate);
+        if (options[0] != '\0')
+          (void) strlcat(options," ",sizeof(options));
+        (void) strlcat(options,arg,sizeof(options));
+      }
     (void) strlcpy(filename,image_info->filename,MaxTextExtent);
     clone_info=CloneImageInfo(image_info);
     if (!AcquireTemporaryFileName(clone_info->filename))
       {
-	DestroyImageInfo(clone_info);
-	ThrowReaderTemporaryFileException(clone_info->filename);
+        DestroyImageInfo(clone_info);
+        ThrowReaderTemporaryFileException(clone_info->filename);
       }
     FormatString(command,delegate_info->commands,antialias,
-		 antialias,density,options,clone_info->filename,
-		 postscript_filename);
+                 antialias,density,options,clone_info->filename,
+                 postscript_filename);
   }
   (void) MagickMonitorFormatted(0,8,&image->exception,RenderPostscriptText,
                                 image->filename);
@@ -386,11 +415,14 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (IsAccessibleAndNotEmpty(clone_info->filename))
     {
       /*
-	Read Ghostscript output.
+        Read Ghostscript output.
       */
       clone_info->blob=(void *) NULL;
       clone_info->length=0;
       clone_info->magick[0]='\0';
+      clone_info->subimage=0;
+      clone_info->subrange=0;
+      MagickFreeMemory(clone_info->tile);
       image=ReadImage(clone_info,exception);
     }
   (void) LiberateTemporaryFile(postscript_filename);
@@ -399,20 +431,30 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (image == (Image *) NULL)
     {
       if (UndefinedException == exception->severity)
-	ThrowException(exception,DelegateError,PostscriptDelegateFailed,filename);
+        ThrowException(exception,DelegateError,PostscriptDelegateFailed,filename);
     }
   else
     {
       do
-	{
-	  (void) strcpy(image->magick,"PDF");
-	  (void) strlcpy(image->filename,filename,MaxTextExtent);
-	  next_image=SyncNextImageInList(image);
-	  if (next_image != (Image *) NULL)
-	    image=next_image;
-	} while (next_image != (Image *) NULL);
+        {
+          (void) strlcpy(image->magick,"PDF",sizeof(image->magick));
+          (void) strlcpy(image->filename,filename,sizeof(image->filename));
+          next_image=SyncNextImageInList(image);
+          if (next_image != (Image *) NULL)
+            image=next_image;
+        } while (next_image != (Image *) NULL);
       while (image->previous != (Image *) NULL)
-	image=image->previous;
+        image=image->previous;
+      if (image_info->subimage != 0)
+        {
+          unsigned long
+            scene = image_info->subimage;
+
+          for (next_image=image;
+               next_image != (Image *) NULL;
+               next_image=next_image->next)
+            next_image->scene = scene++;
+        }
     }
   return(image);
 }
@@ -554,6 +596,18 @@ static char *EscapeParenthesis(const char *text)
   return(buffer);
 }
 
+#define ThrowPDFWriterException(code_,reason_,image_) \
+  {                                                   \
+    MagickFreeMemory(fax_blob);                       \
+    MagickFreeMemory(xref);                           \
+    ThrowWriterException(code_,reason_,image_);       \
+  }
+#define ThrowPDFWriterException2(code_,reason_,image_)  \
+  {                                                     \
+    MagickFreeMemory(fax_blob);                         \
+    MagickFreeMemory(xref);                             \
+    ThrowWriterException2(code_,reason_,image_);        \
+  }
 static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
 {
 #define CFormat  "/Filter [ /%.1024s ]\n"
@@ -631,7 +685,10 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     text_size;
 
   ExtendedSignedIntegralType
-    *xref;
+    *xref = (ExtendedSignedIntegralType *) NULL;
+
+  size_t
+    image_list_length;
 
   /*
     Open output image file.
@@ -640,9 +697,10 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  image_list_length=GetImageListLength(image);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,UnableToOpenFile,image);
+    ThrowPDFWriterException(FileOpenError,UnableToOpenFile,image);
 
   /*
     Allocate X ref memory.
@@ -650,7 +708,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   xref=MagickAllocateMemory(ExtendedSignedIntegralType *,
                             2048*sizeof(ExtendedSignedIntegralType));
   if (xref == (ExtendedSignedIntegralType *) NULL)
-    ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+    ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,image);
   (void) memset(xref,0,2048*sizeof(ExtendedSignedIntegralType));
   /*
     Write Documentation Information Dictionary
@@ -668,7 +726,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                time_meridian->tm_mon+1,time_meridian->tm_mday,time_meridian->tm_hour,
                time_meridian->tm_min,time_meridian->tm_sec);
   GetPathComponent(image->filename,BasePath,basename);
-  
+
   FormatString(buffer,"/Title (%.1024s)\n",EscapeParenthesis(basename));
   (void) WriteBlobString(image,buffer);
   FormatString(buffer,"/CreationDate (%.1024s)\n",date);
@@ -723,7 +781,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       MagickReallocMemory(ExtendedSignedIntegralType *,xref,
                           (count+2048)*sizeof(ExtendedSignedIntegralType));
       if (xref == (ExtendedSignedIntegralType *) NULL)
-        ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+        ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,image);
     }
   (void) WriteBlobString(image,"]\n");
   FormatString(buffer,"/Count %lu\n",(count-pages_id)/ObjectsPerImage);
@@ -734,76 +792,76 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   do
     {
       ImageCharacteristics
-	characteristics;
+        characteristics;
 
       CompressionType
-	compression;
+        compression;
 
       /*
-	Analyze image properties.
+        Analyze image properties.
       */
       (void) GetImageCharacteristics(image,&characteristics,
-				     (OptimizeType == image_info->type),
-				     &image->exception);
+                                     (OptimizeType == image_info->type),
+                                     &image->exception);
 
       compression=image->compression;
       if (image_info->compression != UndefinedCompression)
-	{
-	  /*
-	    ImageInfo compression always prevails if it is set.
-	  */
-	  compression=image_info->compression;
-	}
+        {
+          /*
+            ImageInfo compression always prevails if it is set.
+          */
+          compression=image_info->compression;
+        }
       else
-	{
-	  /*
-	    Default to Zip compression unless the image was JPEG
-	    compressed and is not now monochrome or colormapped.
-	  */
-	  if ((JPEGCompression != compression) ||
-	      (characteristics.monochrome) ||
-	      (characteristics.palette))
-	    {
+        {
+          /*
+            Default to Zip compression unless the image was JPEG
+            compressed and is not now monochrome or colormapped.
+          */
+          if ((JPEGCompression != compression) ||
+              (characteristics.monochrome) ||
+              (characteristics.palette))
+            {
 #if defined(HasZLIB)
-	      compression=ZipCompression;
+              compression=ZipCompression;
 #else
-	      compression=LZWCompression;
+              compression=LZWCompression;
 #endif
-	    }
-	}
-      
+            }
+        }
+
       switch (compression)
-	{
+        {
 #if !defined(HasJPEG)
-	case JPEGCompression:
-	  {
-	    /*
-	      If JPEG compression is not supported, then use RLE compression
-	      and report a warning to user.
-	    */
-	    compression=RLECompression;
-	    ThrowException(&image->exception,MissingDelegateError,JPEGLibraryIsNotAvailable,image->filename);
-	    break;
-	  }
+        case JPEGCompression:
+          {
+            /*
+              If JPEG compression is not supported, then use RLE compression
+              and report a warning to user.
+            */
+            compression=RLECompression;
+            ThrowException(&image->exception,MissingDelegateError,JPEGLibraryIsNotAvailable,image->filename);
+            break;
+          }
 #endif
 #if !defined(HasZLIB)
-	case ZipCompression:
-	  {
-	    /*
-	      If ZIP compression is not supported, then use RLE compression
-	      and report a warning to user.
-	    */
-	    compression=RLECompression;
-	    ThrowException(&image->exception,MissingDelegateError,ZipLibraryIsNotAvailable,image->filename);
-	    break;
-	  }
+        case ZipCompression:
+          {
+            /*
+              If ZIP compression is not supported, then use RLE compression
+              and report a warning to user.
+            */
+            compression=RLECompression;
+            ThrowException(&image->exception,MissingDelegateError,ZipLibraryIsNotAvailable,image->filename);
+            break;
+          }
 #endif
-	default:
-	  break;
-	}
+        default:
+          break;
+        }
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-			    "%s compression.",
-			    CompressionTypeToString(compression));
+                            "%s compression.",
+                            CompressionTypeToString(compression));
 
       /*
         Scale image to size of Portable Document page.
@@ -853,22 +911,57 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                                &geometry.width,&geometry.height);
       (void) GetGeometry(page_geometry,&media_info.x,&media_info.y,
                          &media_info.width,&media_info.height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Image Resolution: %gx%g %s",
+                            image->x_resolution,
+                            image->y_resolution,
+                            ResolutionTypeToString(image->units));
       /*
         Scale relative to dots-per-inch.
       */
       dx_resolution=72.0;
       dy_resolution=72.0;
       x_resolution=72.0;
-      (void) strcpy(density,PSDensityGeometry);
+      (void) strlcpy(density,PSDensityGeometry,sizeof(density));
       count=GetMagickDimension(density,&x_resolution,&y_resolution,NULL,NULL);
       if (count != 2)
         y_resolution=x_resolution;
-      if (image_info->density != (char *) NULL)
+      /*
+        Use override resolution information if it appears to be valid.
+      */
+      if ((image_info->density != (char *) NULL) &&
+          ((image_info->units == PixelsPerInchResolution) ||
+           (image_info->units == PixelsPerCentimeterResolution)))
         {
-          count=GetMagickDimension(image_info->density,&x_resolution,&y_resolution,NULL,NULL);
+          count=GetMagickDimension(image_info->density,&x_resolution,
+                                   &y_resolution,NULL,NULL);
           if (count != 2)
             y_resolution=x_resolution;
+          if (image_info->units == PixelsPerCentimeterResolution)
+            {
+              x_resolution *= 2.54;
+              y_resolution *= 2.54;
+            }
         }
+      /*
+        Use image resolution information if it appears to be valid.
+      */
+      else if ((image->x_resolution > 0.0) && (image->y_resolution > 0.0) &&
+               ((image->units == PixelsPerInchResolution) ||
+                (image->units == PixelsPerCentimeterResolution)))
+        {
+          x_resolution = image->x_resolution;
+          y_resolution = image->y_resolution;
+          if (image->units == PixelsPerCentimeterResolution)
+            {
+              x_resolution *= 2.54;
+              y_resolution *= 2.54;
+            }
+        }
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "PDF Resolution: %gx%g DPI",
+                            x_resolution,y_resolution);
+
       x_scale=(geometry.width*dx_resolution)/x_resolution;
       geometry.width=(unsigned long) (x_scale+0.5);
       y_scale=(geometry.height*dy_resolution)/y_resolution;
@@ -959,12 +1052,12 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       FormatString(buffer,"%lu 0 obj\n",object);
       (void) WriteBlobString(image,buffer);
       if ((image->storage_class == DirectClass) || (image->colors > 256))
-        (void) strcpy(buffer,"[ /PDF /Text /ImageC");
+        (void) strlcpy(buffer,"[ /PDF /Text /ImageC",sizeof(buffer));
       else
         if (compression == FaxCompression)
-          (void) strcpy(buffer,"[ /PDF /Text /ImageB");
+          (void) strlcpy(buffer,"[ /PDF /Text /ImageB",sizeof(buffer));
         else
-          (void) strcpy(buffer,"[ /PDF /Text /ImageI");
+          (void) strlcpy(buffer,"[ /PDF /Text /ImageI",sizeof(buffer));
       (void) WriteBlobString(image,buffer);
       (void) WriteBlobString(image," ]\n");
       (void) WriteBlobString(image,"endobj\n");
@@ -986,13 +1079,13 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
             FormatString(buffer,CFormat,"ASCII85Decode");
             break;
           }
-        case JPEGCompression: 
+        case JPEGCompression:
           {
-            FormatString(buffer,CFormat,"DCTDecode"); 
+            FormatString(buffer,CFormat,"DCTDecode");
             if (image->colorspace != CMYKColorspace)
               break;
             (void) WriteBlobString(image,buffer);
-            (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+            (void) strlcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n",sizeof(buffer));
             break;
           }
         case LZWCompression:
@@ -1007,32 +1100,32 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
           }
         case FaxCompression:
           {
-	    char
-	      CCITTParam[4];
+            char
+              CCITTParam[4];
 
-	    ExceptionInfo
-	      exception;
+            ExceptionInfo
+              exception;
 
-	    /*
-	      Try compressing page to Group4 to see if it is
-	      supported, otherwise we will fall back to Group3.
-	    */
-	    (void) strlcpy(CCITTParam,"0",sizeof(CCITTParam));
-	    GetExceptionInfo(&exception);
-	    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-				  "Executing ImageToHuffman2DBlob for CCITT Fax4 ...");
-	    fax_blob=ImageToHuffman2DBlob(image,image_info,&fax_blob_length,
-					  &exception);
-	    if (fax_blob != (unsigned char *) NULL)
-	      {
-		(void) strlcpy(CCITTParam,"-1",sizeof(CCITTParam));
-		(void) LogMagickEvent(CoderEvent,GetMagickModule(),
-				      "ImageToHuffman2DBlob reports success!");
-	      }
-	    DestroyExceptionInfo(&exception);
-            (void) strcpy(buffer,"/Filter [ /CCITTFaxDecode ]\n");
+            /*
+              Try compressing page to Group4 to see if it is
+              supported, otherwise we will fall back to Group3.
+            */
+            (void) strlcpy(CCITTParam,"0",sizeof(CCITTParam));
+            GetExceptionInfo(&exception);
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "Executing ImageToHuffman2DBlob for CCITT Fax4 ...");
+            fax_blob=ImageToHuffman2DBlob(image,image_info,&fax_blob_length,
+                                          &exception);
+            if (fax_blob != (unsigned char *) NULL)
+              {
+                (void) strlcpy(CCITTParam,"-1",sizeof(CCITTParam));
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                      "ImageToHuffman2DBlob reports success!");
+              }
+            DestroyExceptionInfo(&exception);
+            (void) strlcpy(buffer,"/Filter [ /CCITTFaxDecode ]\n",sizeof(buffer));
             (void) WriteBlobString(image,buffer);
-            (void) strcpy(buffer,"/Interpolate false\n");
+            (void) strlcpy(buffer,"/Interpolate false\n",sizeof(buffer));
             (void) WriteBlobString(image,buffer);
             FormatString(buffer,
                          "/DecodeParms [ << /K %.1024s /BlackIs1 false /Columns %ld /Rows %ld >> ]\n",
@@ -1068,21 +1161,21 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
           ((image_info->type != TrueColorType) &&
            characteristics.grayscale))
         {
-	  /*
-	    Write grayscale output.
-	  */
+          /*
+            Write grayscale output.
+          */
           switch (compression)
             {
             case FaxCompression:
               {
-		/*
-		  Try Group4 first and use Group3 as a fallback.
-		*/
-		if (fax_blob != (unsigned char *) NULL)
-		  {
-		    (void) WriteBlob(image,fax_blob_length,fax_blob);
-		    MagickFreeMemory(fax_blob);
-		  }
+                /*
+                  Try Group4 first and use Group3 as a fallback.
+                */
+                if (fax_blob != (unsigned char *) NULL)
+                  {
+                    (void) WriteBlob(image,fax_blob_length,fax_blob);
+                    MagickFreeMemory(fax_blob);
+                  }
                 else
                   {
                     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -1101,9 +1194,9 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 /*
                   Write image in JPEG format.
                 */
-		jpeg_blob=ImageToJPEGBlob(image,image_info,&length,&image->exception);
+                jpeg_blob=ImageToJPEGBlob(image,image_info,&length,&image->exception);
                 if (jpeg_blob == (unsigned char *) NULL)
-                  ThrowWriterException2(CoderError,image->exception.reason,image);
+                  ThrowPDFWriterException2(CoderError,image->exception.reason,image);
                 (void) WriteBlob(image,length,jpeg_blob);
                 MagickFreeMemory(jpeg_blob);
                 break;
@@ -1117,8 +1210,8 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 length=number_pixels;
                 pixels=MagickAllocateMemory(unsigned char *,length);
                 if (pixels == (unsigned char *) NULL)
-                  ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
-                                       image);
+                  ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,
+                                          image);
                 /*
                   Dump Runlength encoded pixels.
                 */
@@ -1141,7 +1234,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                                                         &image->exception,
                                                         SaveImageText,
                                                         image->filename,
-							image->columns,image->rows);
+                                                        image->columns,image->rows);
                           if (status == False)
                             break;
                         }
@@ -1159,7 +1252,8 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 if (!status)
                   {
                     CloseBlob(image);
-                    return(False);
+                    MagickFreeMemory(xref);
+                    return(MagickFail);
                   }
                 break;
               }
@@ -1188,7 +1282,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                                                         &image->exception,
                                                         SaveImageText,
                                                         image->filename,
-							image->columns,image->rows);
+                                                        image->columns,image->rows);
                           if (status == False)
                             break;
                         }
@@ -1211,9 +1305,9 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 /*
                   Write image in JPEG format.
                 */
-		jpeg_blob=ImageToJPEGBlob(image,image_info,&length,&image->exception);
+                jpeg_blob=ImageToJPEGBlob(image,image_info,&length,&image->exception);
                 if (jpeg_blob == (unsigned char *) NULL)
-                  ThrowWriterException2(CoderError,image->exception.reason,image);
+                  ThrowPDFWriterException2(CoderError,image->exception.reason,image);
                 (void) WriteBlob(image,length,jpeg_blob);
                 MagickFreeMemory(jpeg_blob);
                 break;
@@ -1227,8 +1321,8 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 length=(image->colorspace == CMYKColorspace ? 4 : 3)*number_pixels;
                 pixels=MagickAllocateMemory(unsigned char *,length);
                 if (pixels == (unsigned char *) NULL)
-                  ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
-                                       image);
+                  ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,
+                                          image);
                 /*
                   Dump runoffset encoded pixels.
                 */
@@ -1263,7 +1357,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                                                         &image->exception,
                                                         SaveImageText,
                                                         image->filename,
-							image->columns,image->rows);
+                                                        image->columns,image->rows);
                           if (status == False)
                             break;
                         }
@@ -1320,7 +1414,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                                                         &image->exception,
                                                         SaveImageText,
                                                         image->filename,
-							image->columns,image->rows);
+                                                        image->columns,image->rows);
                           if (status == False)
                             break;
                         }
@@ -1345,7 +1439,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                   length=number_pixels;
                   pixels=MagickAllocateMemory(unsigned char *,length);
                   if (pixels == (unsigned char *) NULL)
-                    ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+                    ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,image);
                   /*
                     Dump Runlength encoded pixels.
                   */
@@ -1366,7 +1460,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                                                           &image->exception,
                                                           SaveImageText,
                                                           image->filename,
-							  image->columns,image->rows);
+                                                          image->columns,image->rows);
                             if (status == False)
                               break;
                           }
@@ -1410,7 +1504,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                                                           &image->exception,
                                                           SaveImageText,
                                                           image->filename,
-							  image->columns,image->rows);
+                                                          image->columns,image->rows);
                             if (status == False)
                               break;
                           }
@@ -1440,7 +1534,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,buffer);
       if (image->colorspace == CMYKColorspace)
         {
-          (void) strcpy(buffer,"/DeviceCMYK\n");
+          (void) strlcpy(buffer,"/DeviceCMYK\n",sizeof(buffer));
         }
       else
         {
@@ -1449,13 +1543,13 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
               ((image_info->type != TrueColorType) &&
                (characteristics.grayscale)))
             {
-              (void) strcpy(buffer,"/DeviceGray\n");
+              (void) strlcpy(buffer,"/DeviceGray\n",sizeof(buffer));
             }
           else
             {
               if ((image->storage_class == DirectClass) || (image->colors > 256) ||
                   (compression == JPEGCompression))
-                (void) strcpy(buffer,"/DeviceRGB\n");
+                (void) strlcpy(buffer,"/DeviceRGB\n",sizeof(buffer));
               else
                 FormatString(buffer,"[ /Indexed /DeviceRGB %u %lu 0 R ]\n",
                              (unsigned int) image->colors-1,object+1);
@@ -1516,7 +1610,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       if (image->next == (Image *) NULL)
         break;
       image=SyncNextImageInList(image);
-      status=MagickMonitorFormatted(scene++,GetImageListLength(image),
+      status=MagickMonitorFormatted(scene++,image_list_length,
                                     &image->exception,SaveImagesText,
                                     image->filename);
       if (status == False)
@@ -1554,7 +1648,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   MagickFreeMemory(xref);
   CloseBlob(image);
   MagickFreeMemory(fax_blob);
-  return(True);
+  return(MagickPass);
 }
 
 #if defined(HasZLIB)
@@ -1596,10 +1690,11 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
 %
 %
 */
+static voidpf ZLIBAllocFunc(voidpf opaque, uInt items, uInt size) MAGICK_FUNC_MALLOC;
 static voidpf ZLIBAllocFunc(voidpf opaque, uInt items, uInt size)
 {
   ARG_NOT_USED(opaque);
-  return MagickMallocCleared((size_t) items*size);
+  return MagickMallocCleared(MagickArraySize(items,size));
 }
 static void ZLIBFreeFunc(voidpf opaque, voidpf address)
 {
