@@ -127,6 +127,60 @@ static unsigned int IsXPM(const unsigned char *magick,const size_t length)
 %
 */
 
+static char **StringToListMod(char *text)
+{
+  char
+    **textlist;
+
+  register char
+    *p;
+
+  register size_t
+    i;
+
+  size_t
+    lines;
+
+  if (text == (char *) NULL)
+    return((char **) NULL);
+
+  /*
+    Convert string to an ASCII list, modifying the input string.
+  */
+  lines=1;
+  for (p=text; *p != '\0'; p++)
+    if (*p == '\n')
+      lines++;
+  textlist=MagickAllocateMemory(char **,(lines+1)*sizeof(char *));
+  if (textlist == (char **) NULL)
+    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+                      UnableToConvertText);
+  i=0;
+  textlist[i]=text;
+  for (p=text; *p != '\0'; p++)
+    {
+      if (*p == '\r')
+        {
+          *p='\0';
+          continue;
+        }
+      if (*p == '\n')
+        {
+          *p='\0';
+          if (*(p+1) != '\0')
+            {
+              i++;
+              textlist[i]=p+1;
+            }
+        }
+    }
+  i++;
+  textlist[i]=(char *) NULL;
+  return(textlist);
+}
+
+static char *ParseColor(char *data) MAGICK_FUNC_PURE;
+
 static char *ParseColor(char *data)
 {
 #define NumberTargets  6
@@ -172,9 +226,6 @@ do { \
     for (i=0; i < (long) image->colors; i++) \
       MagickFreeMemory(keys[i]); \
   MagickFreeMemory(keys); \
-  if (textlist) \
-    for (i=0; textlist[i] != (char *) NULL; i++) \
-      MagickFreeMemory(textlist[i]); \
   MagickFreeMemory(textlist); \
   MagickFreeMemory(xpm_buffer); \
   ThrowReaderException(code_,reason_,image_); \
@@ -315,21 +366,37 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         ThrowXPMReaderException(CorruptImageError,CorruptImage,image);
       }
   }
-  textlist=StringToList(xpm_buffer);
-  MagickFreeMemory(xpm_buffer);
+  /*
+    Scan for non-white space binary control codes and reject file if
+    they are present.
+  */
+  for (p=xpm_buffer; *p != '\0'; p++)
+    if (((unsigned char) *p < 32) && !isspace((int)(unsigned char) (*p)))
+      break;
+  if (*p != '\0')
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Binary control codes error");
+        ThrowXPMReaderException(CorruptImageError,CorruptImage,image);
+    }
+  textlist=StringToListMod(xpm_buffer);
   if (textlist == (char **) NULL)
     ThrowXPMReaderException(ResourceLimitError,MemoryAllocationFailed,image);
 #if 0
   if (image->logging)
     {
+      for (i=0; textlist[i] != (char *) NULL; i++)
+        { };
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "TextList has %lu entries", i);
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                             "TextList");
       for (i=0; textlist[i] != (char *) NULL; i++)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                              "    %lu: %s", i, textlist[i]);
+                              "    %lu: \"%s\"", i, textlist[i]);
     }
 #endif
-  
+
   /*
     Initialize image structure.
   */
@@ -455,9 +522,8 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   for (i=0; i < (long) image->colors; i++)
     MagickFreeMemory(keys[i]);
   MagickFreeMemory(keys);
-  for (i=0; textlist[i] != (char *) NULL; i++)
-    MagickFreeMemory(textlist[i]);
   MagickFreeMemory(textlist);
+  MagickFreeMemory(xpm_buffer);
   CloseBlob(image);
   return(image);
 }
@@ -829,7 +895,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
     if (QuantumTick(y,picon->rows))
       if (!MagickMonitorFormatted(y,picon->rows,&image->exception,
                                   SaveImageText,image->filename,
-				  image->columns,image->rows))
+                                  image->columns,image->rows))
         break;
   }
   DestroyImage(picon);
@@ -1059,7 +1125,7 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
     if (QuantumTick(y,image->rows))
       if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                   SaveImageText,image->filename,
-				  image->columns,image->rows))
+                                  image->columns,image->rows))
         break;
   }
   (void) WriteBlobString(image,"};\n");
